/**
  ******************************************************************************
  * @file    stm8l1526_glass_lcd.c
  * @author  MCD Application Team
  * @version V0.0.1
  * @date    08/21/2009
  * @brief   This file includes driver for the glass LCD Module mounted on 
  *          STM8L1526 LPdemo board.
  ******************************************************************************
  * @copy
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2009 STMicroelectronics</center></h2>
  **/ 

// Includes ------------------------------------------------------------------
#include "stm8l1526_glass_lcd.h"

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
// Private variables ---------------------------------------------------------
const uint16_t mask[4]={0xF000,0x0F00,0x00F0,0x000F};
const uint8_t shift[4]={12,8,4,0};

		
__IO uint8_t digit[4];     /* Digit frame buffer */
		
/*  =========================================================================
                                 LCD MAPPING
    =========================================================================

               A
       _  ----------
	COL |_| |\   |J  /|
         F| H  |  K |B
		   _  |  \ | /  |
  COL |_| --G-- --M--
          |   /| \  |
         E|  Q |  N |C
       _  | /  |P  \|   
    DP|_| -----------  
               D         

 An LCD character coding is based on the following matrix:
 { P , E , F , COL }
 { D , Q , G , H   }
 { N , C , B , A 	 }
 { DP, M , K , J   }


 The character A for example is:
  -------------------------------
   { 0 , 1 , 1 , 0 }
   { 0 , 0 , 1 , 0 }
   { 0 , 1 , 1 , 1 }
   { 0 , 1 , 0 , 0 }
  -------------------
  =  0   D   7   4   hex
  
  => 'A' = 0x0D74
*/

/*  =========================================================================
                       LETTERS AND NUMBERS MAPPING DEFINITION
    ========================================================================= */

// Letters and number map of  LCD 
const uint16_t LetterMap[26]=
    {
        // A      B      C      D      E      F      G      H      I  
        0x0D74,0x3C4C,0x2114,0x344C,0x2934,0x0934,0x2D34,0x0D70,0x0440,
        // J      K      L      M      N      O      P      Q      R  
        0x2540,0x41B0,0x2110,0x05D2,0x4552,0x2554,0x0974,0x6554,0x4974,
        // S      T      U      V      W      X      Y      Z  
        0x2C34,0x100C,0x2550,0x0390,0x4750,0x4282,0x0282,0x2284
    };
const uint16_t uMap = 0x2500;
const uint16_t mMap = 0x1D20;
		
const uint16_t NumberMap[10]=
    {
        // 0      1      2      3      4      5      6      7      8      9  
        0x2554,0x0440,0x2964,0x2C64,0x0C70,0x2C34,0x2D34,0x0444,0x2D74,0x2C74
    };

// Private macro -------------------------------------------------------------
// Private function prototypes -----------------------------------------------
static void Convert(uint8_t* c,Point_Typedef point,Column_Typedef column);
static void delay(__IO uint32_t nCount);
// Private functions ---------------------------------------------------------


/**
  * @brief  Configures the LCD GLASS relative GPIO port IOs and LCD peripheral.
  * @param  None
  * @retval None
  */
void LCD_GLASS_Init(void)
{
	
    // Enable LCD/RTC clock 
    CLK_PeripheralClockConfig(CLK_Peripheral_RTC, ENABLE);
    CLK_PeripheralClockConfig(CLK_Peripheral_LCD, ENABLE);
		#ifdef USE_LSE
		  CLK_RTCClockConfig(CLK_RTCCLKSource_LSE, CLK_RTCCLKDiv_1);
		#else
		  CLK_RTCClockConfig(CLK_RTCCLKSource_LSI, CLK_RTCCLKDiv_1);
		#endif

    // Initialize the LCD 
    LCD_Init(LCD_Prescaler_8, LCD_Divider_16, LCD_Duty_1_4, 
		         LCD_Bias_1_3, LCD_VoltageSource_Internal);

    // Mask register
    LCD_PortMaskConfig(LCD_PortMaskRegister_0, 0xFF);
    LCD_PortMaskConfig(LCD_PortMaskRegister_1, 0xFF);
    LCD_PortMaskConfig(LCD_PortMaskRegister_2, 0xFF);
    LCD_PortMaskConfig(LCD_PortMaskRegister_3, 0xFF);

    LCD_ContrastConfig(LCD_Contrast_2V9);
    LCD_DeadTimeConfig(LCD_DeadTime_0);
    LCD_PulseOnDurationConfig(LCD_PulseOnDuration_2);

    LCD_Cmd(ENABLE); /*!< Enable LCD peripheral */
}
void LCD_Test(void)
{
	int i=0;
	LCD_GLASS_DisplayString("LCDTST");
	delay_ms(100);
	for (i=0; i<14; i++)
		LCD->RAM[i] = 0xFF;
	delay_ms(100);
	for (i=0; i<14; i++)
		LCD->RAM[i] = 0xAA;	
	delay_ms(100);
	for (i=0; i<14; i++)
		LCD->RAM[i] = 0x55;	
	delay_ms(100);
	for (i=0; i<14; i++)
		LCD->RAM[i] = 0x0;	
	
	for (i=0; i<7; i++)
	{
		LCD_GLASS_WriteChar("0", 0, 0, i);
		delay_ms(30);
		LCD_GLASS_Clear();			
		LCD_GLASS_WriteChar("0", 1, 0, i);
		delay_ms(30);
		LCD_GLASS_Clear();
		LCD_GLASS_WriteChar("0", 1, 1, i);
		delay_ms(30);
		LCD_GLASS_Clear();			
	}	
	LCD_GLASS_WriteChar("0", 0, 0, 7);
	delay_ms(30);
	LCD_GLASS_Clear();

	
	for (i=0; i<8; i++)		
	{
		LCD_GLASS_WriteChar("X", 0, 0, i);	
		delay_ms(30);	
		LCD_GLASS_Clear();

	}
	LCD_GLASS_DisplayString("PRESS");
	delay_ms(50);
	LCD_GLASS_DisplayString("FUNC");
	delay_ms(50);
	LCD_GLASS_DisplayString("THEN");
	delay_ms(50);
	LCD_GLASS_DisplayString("WKUP");
	
	
}
/**
  * @brief  This function writes a char in the LCD frame buffer.
  * @param  ch: the character to dispaly.
  * @param  point: a point to add in front of char
  *         This parameter can be: POINT_OFF or POINT_ON
  * @param  column: flag indicating if a column has to be add in front
  *         of displayed character.
  *         This parameter can be: COLUMN_OFF or COLUMN_ON.           
  * @param  position: position in the LCD of the caracter to write [0:7]
  * @retval None
  * @par    Required preconditions: The LCD should be cleared before to start the
  *         write operation.  
  */
void LCD_GLASS_WriteChar(uint8_t* ch, Point_Typedef point,
                         Column_Typedef column, uint8_t position)
{
    Convert(ch,point,column);

    switch (position)
    {
        // Position 1 on LCD (Digit1)
    case 1:
		    
        break;

        // Position 2 on LCD (Digit2)
    case 2:
		    LCD->RAM[LCD_RAMRegister_0] &= 0x87;
				LCD->RAM[LCD_RAMRegister_0] |=  (uint8_t)(digit[0]<<3);
		    LCD->RAM[LCD_RAMRegister_3] &= 0x3F;
				LCD->RAM[LCD_RAMRegister_3] |=  (uint8_t) (digit[1]<<6);
				LCD->RAM[LCD_RAMRegister_4] &= 0xFC;
				LCD->RAM[LCD_RAMRegister_4] |=  (uint8_t)(digit[1]>>2);
		    LCD->RAM[LCD_RAMRegister_7] &= 0xC3;
				LCD->RAM[LCD_RAMRegister_7] |=  (uint8_t)(digit[2]<<2);
				LCD->RAM[LCD_RAMRegister_10] &= 0x3F;
				LCD->RAM[LCD_RAMRegister_10] |=  (uint8_t)(digit[3]<<6);
				LCD->RAM[LCD_RAMRegister_11] &= 0xFC;
				LCD->RAM[LCD_RAMRegister_11] |=  (uint8_t)(digit[3]>>2);
				
        break;

        // Position 3 on LCD (Digit3)
    case 3:
		    LCD->RAM[LCD_RAMRegister_0] &= 0x7F;
				LCD->RAM[LCD_RAMRegister_0] |=  (uint8_t)(digit[0]<<7);
				LCD->RAM[LCD_RAMRegister_1] &= 0xF8;
				LCD->RAM[LCD_RAMRegister_1] |=  (uint8_t)(digit[0]>>1);
		    LCD->RAM[LCD_RAMRegister_4] &= 0xC3;
				LCD->RAM[LCD_RAMRegister_4] |=  (uint8_t)(digit[1]<<2);
				LCD->RAM[LCD_RAMRegister_7] &= 0x3F;
				LCD->RAM[LCD_RAMRegister_7] |=  (uint8_t)(digit[2]<<6);
				LCD->RAM[LCD_RAMRegister_8] &= 0xFC;
				LCD->RAM[LCD_RAMRegister_8] |= (uint8_t)(digit[2]>>2);
		    LCD->RAM[LCD_RAMRegister_11] &= 0xC3;
				LCD->RAM[LCD_RAMRegister_11] |= (uint8_t) (digit[3]<<2);
        break;

        // Position 4 on LCD (Digit4)
    case 4:
		    LCD->RAM[LCD_RAMRegister_1] &= 0x87;
				LCD->RAM[LCD_RAMRegister_1] |=  (uint8_t)(digit[0]<<3);
				LCD->RAM[LCD_RAMRegister_4] &= 0x3F;
				LCD->RAM[LCD_RAMRegister_4] |=  (uint8_t) (digit[1]<<6);
				LCD->RAM[LCD_RAMRegister_5] &= 0xFC;
				LCD->RAM[LCD_RAMRegister_5] |=  (uint8_t)(digit[1]>>2);
		    LCD->RAM[LCD_RAMRegister_8] &= 0xC3;
				LCD->RAM[LCD_RAMRegister_8] |=  (uint8_t)(digit[2]<<2);
				LCD->RAM[LCD_RAMRegister_11] &= 0x3F;
				LCD->RAM[LCD_RAMRegister_11] |=  (uint8_t)(digit[3]<<6);
				LCD->RAM[LCD_RAMRegister_12] &= 0xFC;
				LCD->RAM[LCD_RAMRegister_12] |=  (uint8_t)(digit[3]>>2);
        break;

        // Position 5 on LCD (Digit5)
    case 5:
		    LCD->RAM[LCD_RAMRegister_1] &= 0x7F;
				LCD->RAM[LCD_RAMRegister_1] |=  (uint8_t)(digit[0]<<7);
				LCD->RAM[LCD_RAMRegister_2] &= 0xF8;
				LCD->RAM[LCD_RAMRegister_2] |=  (uint8_t)(digit[0]>>1);
		    LCD->RAM[LCD_RAMRegister_5] &= 0xC3;
				LCD->RAM[LCD_RAMRegister_5] |=  (uint8_t)(digit[1]<<2);
				LCD->RAM[LCD_RAMRegister_8] &= 0x3F;
				LCD->RAM[LCD_RAMRegister_8] |=  (uint8_t)(digit[2]<<6);
				LCD->RAM[LCD_RAMRegister_9] &= 0xFC;
				LCD->RAM[LCD_RAMRegister_9] |= (uint8_t)(digit[2]>>2);
		    LCD->RAM[LCD_RAMRegister_12] &= 0xC3;
				LCD->RAM[LCD_RAMRegister_12] |= (uint8_t) (digit[3]<<2);
        break;

        // Position 6 on LCD (Digit6)
    case 6:
				LCD->RAM[LCD_RAMRegister_2] &= 0x87;
				LCD->RAM[LCD_RAMRegister_2] |=  (uint8_t)(digit[0]<<3);
				LCD->RAM[LCD_RAMRegister_5] &= 0x3F;
				LCD->RAM[LCD_RAMRegister_5] |=  (uint8_t) (digit[1]<<6);
				LCD->RAM[LCD_RAMRegister_6] &= 0xFC;
				LCD->RAM[LCD_RAMRegister_6] |=  (uint8_t)(digit[1]>>2);
		    LCD->RAM[LCD_RAMRegister_9] &= 0xC3;
				LCD->RAM[LCD_RAMRegister_9] |=  (uint8_t)(digit[2]<<2);
				LCD->RAM[LCD_RAMRegister_12] &= 0x3F;
				LCD->RAM[LCD_RAMRegister_12] |=  (uint8_t)(digit[3]<<6);
				LCD->RAM[LCD_RAMRegister_13] &= 0xFC;
				LCD->RAM[LCD_RAMRegister_13] |=  (uint8_t)(digit[3]>>2);
        break;

        // Position 7 on LCD (Digit7)
    case 7:
				LCD->RAM[LCD_RAMRegister_2] &= 0x7F;
				LCD->RAM[LCD_RAMRegister_2] |=  (uint8_t)(digit[0]<<7);
				LCD->RAM[LCD_RAMRegister_3] &= 0xF8;
				LCD->RAM[LCD_RAMRegister_3] |=  (uint8_t)(digit[0]>>1);
		    LCD->RAM[LCD_RAMRegister_6] &= 0xC3;
				LCD->RAM[LCD_RAMRegister_6] |=  (uint8_t)(digit[1]<<2);
				LCD->RAM[LCD_RAMRegister_9] &= 0x3F;
				LCD->RAM[LCD_RAMRegister_9] |=  (uint8_t)(digit[2]<<6);
				LCD->RAM[LCD_RAMRegister_10] &= 0xFC;
				LCD->RAM[LCD_RAMRegister_10] |= (uint8_t)(digit[2]>>2);
		    LCD->RAM[LCD_RAMRegister_13] &= 0xC3;
				LCD->RAM[LCD_RAMRegister_13] |= (uint8_t) (digit[3]<<2);
        break;    

    default:
        break;
    }
}

/**
  * @brief  This function writes a char in the LCD RAM.
  * @param  ptr: Pointer to string to display on the LCD Glass.
  * @retval None
  */
void LCD_GLASS_DisplayString(uint8_t* ptr)
{
  uint8_t i = 0x02;

	LCD_GLASS_Clear();
  // Send the string character by character on lCD 
  while ((*ptr != 0) & (i < 8))
  {
    // Display one character on LCD 
    LCD_GLASS_WriteChar(ptr, 0, 0, i);

    // Point on the next character 
    ptr++;

    // Increment the character counter 
    i++;
  }
}




/**
  * @brief  This function Clear the whole LCD RAM.
  * @param  None
  * @retval None
  */
void LCD_GLASS_Clear(void)
{
	 uint8_t counter = 0;
       for (counter=0;counter<0x0E; counter++)
    {
        LCD->RAM[counter] =  LCD_RAM_RESET_VALUE;
    }
}
/**
  * @brief  Display a string in scrolling mode
  * @param  ptr: Pointer to string to display on the LCD Glass.
  * @param  nScroll: Specifies how many time the message will be scrolled
  * @param  ScrollSpeed : Speciifes the speed of the scroll, low value gives
  *         higher speed 
  * @retval None
  * @par    Required preconditions: The LCD should be cleared before to start the
  *         write operation.
  */
void LCD_GLASS_ScrollSentence(uint8_t* ptr, uint8_t ptr_size, uint16_t nScroll, uint16_t ScrollSpeed)
{
    uint8_t Repetition =0;
    uint8_t Char_Nb =0;
    uint8_t* ptr1;
    __IO uint8_t str[6]="";
    ptr1 =ptr;

		if (ptr == 0) return;
		
    LCD_GLASS_DisplayString(ptr1);
    delay(ScrollSpeed);
		
	
    for (Repetition=0; Repetition<nScroll; Repetition++)
    {
			for (Char_Nb=0; Char_Nb<ptr_size; Char_Nb++)
			{
        *(str) =*(ptr1+((Char_Nb+1)%ptr_size));
        *(str+1) =*(ptr1+((Char_Nb+2)%ptr_size));
        *(str+2) =*(ptr1+((Char_Nb+3)%ptr_size));
        *(str+3) =*(ptr1+((Char_Nb+4)%ptr_size));
        *(str+4) =*(ptr1+((Char_Nb+5)%ptr_size));
        *(str+5) =*(ptr1+((Char_Nb+6)%ptr_size));
        LCD_GLASS_Clear();
        LCD_GLASS_DisplayString(str);
        delay(ScrollSpeed);
				
			}	
    }
		RTC->CR2 = 0x0;
}
void LCD_GLASS_WriteTime(char a, uint8_t posi, Column_Typedef column)
{
	switch (a)
    {
		// Position 0 on LCD (Digit1)
    case 0:
		 LCD_GLASS_WriteChar("0", 0, column, posi);
		 break;
		case 1:
		 LCD_GLASS_WriteChar("1", 0, column, posi);
		 break;
		case 2:
		 LCD_GLASS_WriteChar("2", 0, column, posi);
		 break;
		case 3:
		 LCD_GLASS_WriteChar("3", 0, column, posi);
		 break;
		case 4:
		 LCD_GLASS_WriteChar("4", 0, column, posi);
		 break;
		case 5:
		 LCD_GLASS_WriteChar("5", 0, column, posi);
		 break;
		case 6:
		 LCD_GLASS_WriteChar("6", 0, column, posi);
		 break;
		case 7:
		 LCD_GLASS_WriteChar("7", 0, column, posi);
		 break;
		case 8:
		 LCD_GLASS_WriteChar("8", 0, column, posi);
		 break;
		case 9:
		 LCD_GLASS_WriteChar("9", 0, column, posi); 
		 break;
		default:
		 LCD_GLASS_WriteChar("E", 0, 0, posi);
		 break;
    }
}
/**
  * @brief  Converts an ascii char to the a LCD digit (previous coding).
  * @param  c: a char to display.
  * @param  point: a point to add in front of char
  *         This parameter can be: POINT_OFF or POINT_ON
  * @param  column : flag indicating if a column has to be add in front
  *         of displayed character.
  *         This parameter can be: COLUMN_OFF or COLUMN_ON.   
  * @retval None
  */
static void Convert(uint8_t* c,Point_Typedef point,Column_Typedef column)
{
    uint16_t ch = 0 , tmp = 0;
    uint8_t i;
    
    // The character c is a letter in upper case
    if ((*c<0x5B)&(*c>0x40))
    {
        ch = LetterMap[*c-0x41];
    }
		
		if (*c == 0x75 ) ch = uMap;
			
		if (*c == 0x6D) ch = mMap;
		
    // The character c is a number
    if ((*c<0x3A)&(*c>0x2F))
    {
        ch = NumberMap[*c-0x30];
    }
    // The character c is a space character 
    if (*c == 0x20)
    {
        ch =0x00;
    }
    // Set the DP seg in the character that can be displayed if the point is on 
    if (point == POINT_ON)
    {
        ch|=0x8000;
    }

    // Set the X seg in the character that can be displayed if the column is on 
    if (column == COLUMN_ON)
    {
        ch |= 0x0001;
    }

    for (i = 0;i < 4; i++)
    {
      tmp = ch & mask[i];
      digit[i] =(uint8_t)(tmp >> (uint8_t)shift[i]);
    }
}
/**
  * @brief  Inserts a delay time.
  * @param  nCount: specifies the delay time length.
  * @retval None
  */
static void delay(__IO uint32_t nCount)
{
  __IO uint32_t index = 0; 
  for(index = (0x40 * nCount); index != 0; index--)
  {
  }    
}
